/*
	NGR Converter - National Grid Reference to Latitude and Longitude
 	Copyright © 2005-2019 Harry Whitfield

 	This program is free software; you can redistribute it and/or modify it
 	under the terms of the GNU General Public License as published by the
 	Free Software Foundation; either version 2 of the License, or (at your
 	option) any later version.

 	This program is distributed in the hope that it will be useful, but
 	WITHOUT ANY WARRANTY; without even the implied warranty of
 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 	General Public License for more details.

 	You should have received a copy of the GNU General Public License along
 	with this program; if not, write to the Free Software Foundation, Inc.,
 	51 Franklin St, Fifth Floor, Boston, MA	 02110-1301	 USA

 	NGR Converter - browser version 2.1
 	29 April, 2019
 	Copyright © 2005-2019 Harry Whitfield
 	mailto:g6auc@arrl.net
*/

/*jslint browser, bitwise */

/*global GridRef, numericRefsPref, prec2, figuresPref, parseLatOrLong, prec3, latLongPref,
	prec4, prec1, fnc, fnf
*/

/*property
	PI, eastings, floor, indexOf, isNaN, length, northings, round, split,
	substring, toFixed, toString, toUpperCase
*/

function gridRefOfLetterPair(pair) {
	"use strict";
	var error;
	var first;
	var e1;
	var n1;
	var idx;
	var eastings;
	var northings;
	var second;
	var valid;
	var idx2;
	var x;
	var y;

	pair = pair.toUpperCase();
	error = new GridRef(NaN, NaN);
	first = "HNSOT";
	e1 = [0, 0, 0, 500000, 500000];
	n1 = [1000000, 500000, 0, 500000, 0];
	idx = first.indexOf(pair[0]);
	if (idx < 0) {
		return error;
	}
	eastings = e1[idx];
	northings = n1[idx];
	second = "VWXYZQRSTULMNOPFGHJKABCDE";
	valid = ["PTUYZ", "ABCDFGHJKLMNORSTUWXYZ", "CDEGHJKMNOPRSTUVWXYZ", "V", "AFGLMQRV"];
	idx2 = valid[idx].indexOf(pair[1]);
	if (idx2 < 0) {
		return error;
	}
	idx = second.indexOf(pair[1]);
	if (idx < 0) {
		return error;
	} // should not happen!
	x = idx % 5;
	eastings += 100000 * x;
	y = Math.round((idx - x) / 5);
	northings += 100000 * y;
	//print('gridRefOfLetterPair:' + eastings + ',' + northings);
	return new GridRef(eastings, northings);
}

function letterPairOfGridRef(gridRef) {
	"use strict";
	var first;
	var pair;
	var second;
	var g;
	var eastings = Math.floor(gridRef.eastings);
	var northings = Math.floor(gridRef.northings);
	var idx = Math.floor(eastings / 500000);
	var idy = Math.floor(northings / 500000);

	//print('1:letterPairOfGridRef:' + idx + ',' + idy);
	if ((idx < 0) || (idx > 1)) {
		return "";
	}
	if ((idy < 0) || (idx > 2)) {
		return "";
	}

	first = [
		["S", "N", "H"],
		["T", "O", "J"]
	];
	pair = first[idx][idy];

	eastings -= 500000 * idx;
	northings -= 500000 * idy;

	idx = Math.floor(eastings / 100000);
	idy = Math.floor(northings / 100000);

	//print('2:letterPairOfgridRef:' + idx + ',' + idy);
	second = "VWXYZQRSTULMNOPFGHJKABCDE";

	pair += second[5 * idy + idx];
	//print('pair=' + pair);
	g = gridRefOfLetterPair(pair);
	//print('g.eastings=' + g.eastings);
	if (Number.isNaN(g.eastings)) {
		return "";
	}
	return pair;
}

function isRef(data) {
	"use strict";
	var first = "HNSOT";
	var second = "VWXYZQRSTULMNOPFGHJKABCDE";
	var digit = "0123456789";
	var idx;
	var i;

	data = data.toUpperCase();
	idx = first.indexOf(data[0]);

	if (idx < 0) {
		return false;
	}
	idx = second.indexOf(data[1]);
	if (idx < 0) {
		return false;
	}
	if ((data.length % 2) !== 0) {
		return false;
	}
	if ((data.length < 4) || (data.length > 12)) {
		return false;
	}

	i = 2;
	while (i < data.length) {
		idx = digit.indexOf(data[i]);
		if (idx < 0) {
			return false;
		}
		i += 1;
	}
	return true;
}

function get_ref(data) { // input grid reference
	"use strict";
	var pair;
	var rest;
	var len;
	var eastings;
	var northings;
	var g;
	var item;
	var e;
	var n;


	if (isRef(data)) {
		pair = data.substring(0, 2);
		rest = data.substring(2);
		len = rest.length >> 1;
		eastings = rest.substring(0, len);
		northings = rest.substring(len);

		while (eastings.length < 5) {
			eastings += "0";
		}
		while (northings.length < 5) {
			northings += "0";
		}

		eastings = Number(eastings);
		northings = Number(northings);

		g = gridRefOfLetterPair(pair);

		eastings += g.eastings;
		northings += g.northings;

		return new GridRef(eastings, northings);
	}

	item = data.split(",", 2); // 400000, 100000
	e = Number(item[0]);
	if (Number.isNaN(e)) {
		return new GridRef(NaN, NaN);
	}
	if ((e < 0) || (e >= 700000)) {
		return new GridRef(NaN, NaN);
	}

	n = Number(item[1]);
	if (Number.isNaN(n)) {
		return new GridRef(NaN, NaN);
	}
	if ((n < 0) || (e >= 1300000)) {
		return new GridRef(NaN, NaN);
	}

	return new GridRef(e, n);
}

function put_ref(g) {
	"use strict";
	var eastings = g.eastings;
	var northings = g.northings;
	var figs;
	var pair;
	var tmp;

	if (numericRefsPref === "1") {
		return eastings.toFixed(prec2) + "," + northings.toFixed(prec2);
	}

	tmp = Math.floor(eastings) % 100000;
	eastings = tmp.toString();
	tmp = Math.floor(northings) % 100000;
	northings = tmp.toString();

	while (eastings.length < 5) {
		eastings = "0" + eastings;
	}
	while (northings.length < 5) {
		northings = "0" + northings;
	}

	figs = Number(figuresPref) >> 1;

	eastings = eastings.substring(0, figs);
	northings = northings.substring(0, figs);

	pair = letterPairOfGridRef(g);

	if (pair !== "") {
		return pair + eastings + northings;
	}
	return "Invalid Reference";
}

function get_ang(data, dmax) {	// , NE, SW)
	"use strict";
	switch (dmax) {
	case 90:
		return parseLatOrLong(data, true, false, " ") * Math.PI / 180.0;
	case 180:
		return parseLatOrLong(data, false, false, " ") * Math.PI / 180.0;
	default:
		return NaN;
	}
}

/*
function Xget_ang(data, dmax, NE, SW) { // input angle
	var item,
		a,
		d,
		m,
		s,
		t;

	//print("get_ang: " + data + ', ' + dmax + ', ' + NE + ', ' + SW);
	item = data.split(' ', 4); // 49 45 58.2689 N	or	7 33 23.2145 W
	//print("item: " + item);
	if (item.length < 2) {
		return NaN;
	}

	a = item[item.length - 1];
	a = a.toUpperCase();
	if ((a !== NE) && (a !== SW)) {
		return NaN;
	}

	d = Number(item[0]);
	if (Number.isNaN(d)) {
		return NaN;
	}
	if ((d < 0) || (d >= dmax)) {
		return NaN;
	}
	if (item.length === 2) {
		m = 0;
		s = 0;
	} else {
		if (d !== Math.floor(d)) {
			return NaN;
		}
		m = Number(item[1]);
		if (Number.isNaN(m)) {
			return NaN;
		}
		if ((m < 0) || (m >= 60)) {
			return NaN;
		}
		if (item.length === 3) {
			s = 0;
		} else {
			if (m !== Math.floor(m)) {
				return NaN;
			}
			s = Number(item[2]);
			if (Number.isNaN(s)) {
				return NaN;
			}
			if ((s < 0) || (s >= 60)) {
				return NaN;
			}
		}
	}

	t = d + m / 60.0 + s / 3600.0;
	if (a === SW) {
		t = -t;
	}

	return t * Math.PI / 180.0;
}

function XXget_ang(data, dmax, NE, SW) { // input angle
	var item, d, m, s, a, t;

	item = data.split(' ', 4); // 49 45 58.2689 N	or	7 33 23.2145 W
	d = Number(item[0]);
	if (Number.isNaN(d)) { return NaN; }
	m = Number(item[1]);
	if (Number.isNaN(m)) { return NaN; }
	s = Number(item[2]);
	if (Number.isNaN(s)) { return NaN; }
	a = item[3];
	if (a === undefined) { return NaN; }

	if ((d < 0) || (d >= dmax) || (d !== Math.floor(d))) { return NaN; }
	if ((m < 0) || (m >= 60) || (m !== Math.floor(m))) { return NaN; }
	if ((s < 0) || (s >= 60)) { return NaN; }

	a = a.toUpperCase();
	if ((a !== NE) && (a !== SW)) { return NaN; }

	t = d + m / 60.0 + s / 3600.0;
	if (a === SW) { t = -t; }

	return t * Math.PI / 180.0;
}
*/

function put_ang(st, k) { // output angle
	"use strict";
	var d;
	var m;
	var s;
	var sign;

	sign = "";
	if (k < 0) {
		k = -k;
		sign = "-";
	}
	k = 180.0 * k / Math.PI;
	d = Math.floor(k);
	k = 60.0 * (k - d);
	m = Math.floor(k);
	s = 60.0 * (k - m);
	return st + sign + d + " " + m + " " + s.toFixed(prec3);
}

function put_lat(k) { // output angle
	"use strict";
	var d;
	var m;
	var s;
	var ns;

	ns = "N";
	if (k < 0) {
		k = -k;
		ns = "S";
	}
	k = 180.0 * k / Math.PI;
	if (latLongPref === "1") {
		d = k.toFixed(prec4);
		return d + " " + ns;
	}
	d = Math.floor(k);
	k = 60.0 * (k - d);
	m = Math.floor(k);
	s = 60.0 * (k - m);
	return d + " " + m + " " + s.toFixed(prec1) + " " + ns;
}

function put_long(k) { // output angle
	"use strict";
	var d;
	var m;
	var s;
	var ew;

	ew = "E";
	if (k < 0) {
		k = -k;
		ew = "W";
	}
	k = 180.0 * k / Math.PI;
	if (latLongPref === "1") {
		d = k.toFixed(prec4);
		return d + " " + ew;
	}
	d = Math.floor(k);
	k = 60.0 * (k - d);
	m = Math.floor(k);
	s = 60.0 * (k - m);
	return d + " " + m + " " + s.toFixed(prec1) + " " + ew;
}

function put_convergence(k, l) {
	"use strict";
	return put_ang("", fnc(k, l));
}

function put_scale(k, l) {
	"use strict";
	return fnf(k, l);
}
