/*
	NGR Converter - National Grid Reference to Latitude and Longitude
 	Copyright © 2005-2019 Harry Whitfield

 	This program is free software; you can redistribute it and/or modify it
 	under the terms of the GNU General Public License as published by the
 	Free Software Foundation; either version 2 of the License, or (at your
 	option) any later version.

 	This program is distributed in the hope that it will be useful, but
 	WITHOUT ANY WARRANTY; without even the implied warranty of
 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 	General Public License for more details.

 	You should have received a copy of the GNU General Public License along
 	with this program; if not, write to the Free Software Foundation, Inc.,
 	51 Franklin St, Fifth Floor, Boston, MA	 02110-1301	 USA

 	NGR Converter - browser version 2.1
 	29 April, 2019
 	Copyright © 2005-2019 Harry Whitfield
 	mailto:g6auc@arrl.net
*/

/*jslint browser */

/*global Position */

/*property
    PI, charCodeAt, floor, fromCharCode, length, toUpperCase
*/


var lb = "AA00AA00";
var ub = "RR99XX99";

function get_locator(s) { // return latitude and longitude given locator
    "use strict";
    var loc = [];
	var i;
	var c;
	var kk;
	var ll;

    if (s.length === 6) {
        s += "55";
    }
    if (s.length !== 8) {
        return new Position(NaN, NaN);
    }

    i = 0;
    while (i < 8) {
        c = s[i];
        c = c.toUpperCase();
        if ((lb[i] <= c) && (c <= ub[i])) {
            loc[i] = c.charCodeAt(0) - lb[i].charCodeAt(0);
        } else {
            return new Position(NaN, NaN);
        }
        i += 1;
    }

    kk = (10.0 * loc[1] + loc[3] - 90.0 + (100.0 * loc[5] + 10.0 * loc[7] + 5.0) / 2400.0) * Math.PI / 180.0;
    ll = (10.0 * loc[0] + loc[2] - 90.0 + (100.0 * loc[4] + 10.0 * loc[6] + 5.0) / 2400.0) * Math.PI / 90.0;

    return new Position(kk, ll);
}

function put_locator(k, l) { // return locator given latitude and longitude
    "use strict";
    var d;
	var e;
	var loc = [];
	var s;
	var i;
	var res;

    d = 180.0 * k / Math.PI + 90.0;
    loc[1] = Math.floor(d / 10.0);
    d = d - 10.0 * loc[1];
    loc[3] = Math.floor(d);
    d = d - loc[3];
    loc[5] = Math.floor(24.0 * d);
    d = 24.0 * d - loc[5];
    loc[7] = Math.floor(10.0 * d);

    e = 90.0 * l / Math.PI + 90.0;
    loc[0] = Math.floor(e / 10.0);
    e = e - 10.0 * loc[0];
    loc[2] = Math.floor(e);
    e = e - loc[2];
    loc[4] = Math.floor(24.0 * e);
    e = 24.0 * e - loc[4];
    loc[6] = Math.floor(10.0 * e);

    s = [];
    i = 0;
    while (i < 8) {
        s[i] = String.fromCharCode(loc[i] + lb[i].charCodeAt(0));
        i += 1;
    }

    if (s[1] === "S") {
        s[1] = "R";
        s[3] = "9";
        s[5] = "X";
        s[7] = "9";
    }
    if (s[0] === "S") {
        s[0] = "A";
    }

    res = "";
    i = 0;
    while (i < 8) {
        res += s[i];
        i += 1;
    }
    return res;
}
